#Imports
from PIL import Image, ImageFilter  
# from scipy.ndimage.filters import gaussian_filter
import numpy as np

class BlurImage(object):
    '''
        Applies Gaussian Blur on the image.
    '''

    def __init__(self, radius):
        '''
            Arguments:
            radius (int): radius to blur
        '''

        # Write your code here
        self.radius = radius        

    def __call__(self, image):
        '''
            Arguments:
            image (numpy array or PIL Image)

            Returns:
            image (numpy array or PIL Image)
        '''

        # Write your code here
        # return gaussian_filter(image , sigma = self.radius)
        return np.array(Image.fromarray(image).filter(ImageFilter.GaussianBlur(radius=self.radius)))

# blurrer = BlurImage(2)

# img = Image.open("/home/anish/Software Development Lab/Assignment-3/CS29006_SW_Lab_Spr2021/Python_DS_Assignment/AssignmentQs2/sample_imgs/bbox.png")
# blurredImage =Image.fromarray(blurrer(np.array(img)))
# blurredImage.show()



#Imports
from PIL import Image, ImageOps
import numpy as np

class FlipImage(object):
    '''
        Flips the image.
    '''

    def __init__(self, flip_type='horizontal'):
        '''
            Arguments:
            flip_type: 'horizontal' or 'vertical' Default: 'horizontal'
        '''
        self.flip_type = flip_type
        # Write your code here

        
    def __call__(self, image):
        '''
            Arguments:
            image (numpy array or PIL image)

            Returns:
            image (numpy array or PIL image)
        '''
        # Write your code here
        if self.flip_type == 'horizontal':
            # print(25)
            return np.array(ImageOps.mirror(Image.fromarray(image)))
        elif self.flip_type == 'vertical':
            # print(35)
            return np.array(ImageOps.flip(Image.fromarray(image)))
       

# img = Image.open("/home/anish/Software Development Lab/Assignment-3/CS29006_SW_Lab_Spr2021/Python_DS_Assignment/AssignmentQs2/sample_imgs/bbox.png")
        

# horizontalFlipper = FlipImage('horizontal')
# verticalFlipper = FlipImage('vertical')

# flippedImage = Image.fromarray(horizontalFlipper(np.array(img)))
# flippedImage.show()


# flippedImage = Image.fromarray(verticalFlipper(np.array(img)))
# flippedImage.show()


#Imports
from PIL import Image
import numpy as np

class RescaleImage(object):
    '''
        Rescales the image to a given size.
    '''

    def __init__(self, output_size):
        '''
            Arguments:
            output_size (tuple or int): Desired output size. If tuple, output is
            matched to output_size. If int, smaller of image edges is matched
            to output_size keeping aspect ratio the same.
        '''

        # Write your code here
        self.output_size = output_size

    def __call__(self, image):
        '''
            Arguments:
            image (numpy array or PIL image)

            Returns:
            image (numpy array or PIL image)

            Note: You do not need to resize the bounding boxes. ONLY RESIZE THE IMAGE.
        '''

        # Write your code here
        if isinstance(self.output_size,int):
            w,h = Image.fromarray(image).size
            aspect = w/h
            if w<h:
                w = self.output_size
                h = self.output_size * 1/aspect
            else :
                h = self.output_size
                w = self.output_size * aspect
            return np.array(Image.fromarray(image).resize((int(w),int(h))))
        elif isinstance(self.output_size,tuple):
            return np.array(Image.fromarray(image).resize(self.output_size))
        elif(isinstance(self.output_size, float)):
            w,h = Image.fromarray(image).size
            w *= self.output_size
            h *= self.output_size
            return np.array(Image.fromarray(image).resize((int(w),int(h))))

# img = Image.open("/home/anish/Software Development Lab/Assignment-3/CS29006_SW_Lab_Spr2021/Python_DS_Assignment/AssignmentQs2/sample_imgs/bbox.png")
# print(img.size)

# rescaller = RescaleImage(250)
# rescaledImage = Image.fromarray(rescaller(np.array(img)))
# print(rescaledImage.size)
# rescaledImage.show()


# rescaller = RescaleImage((100,100))
# RescaledImage = Image.fromarray(rescaller(np.array(img)))
# print(RescaledImage.size)
# RescaledImage.show()



import torch
import torch.nn as nn
import torchvision
from torchvision.models.detection.faster_rcnn import FastRCNNPredictor

# Class id to name mapping
COCO_INSTANCE_CATEGORY_NAMES = [
    '__background__', 'person', 'bicycle', 'car', 'motorcycle', 'airplane', 'bus',
    'train', 'truck', 'boat', 'traffic light', 'fire hydrant', 'N/A', 'stop sign',
    'parking meter', 'bench', 'bird', 'cat', 'dog', 'horse', 'sheep', 'cow',
    'elephant', 'bear', 'zebra', 'giraffe', 'N/A', 'backpack', 'umbrella', 'N/A', 'N/A',
    'handbag', 'tie', 'suitcase', 'frisbee', 'skis', 'snowboard', 'sports ball',
    'kite', 'baseball bat', 'baseball glove', 'skateboard', 'surfboard', 'tennis racket',
    'bottle', 'N/A', 'wine glass', 'cup', 'fork', 'knife', 'spoon', 'bowl',
    'banana', 'apple', 'sandwich', 'orange', 'broccoli', 'carrot', 'hot dog', 'pizza',
    'donut', 'cake', 'chair', 'couch', 'potted plant', 'bed', 'N/A', 'dining table',
    'N/A', 'N/A', 'toilet', 'N/A', 'tv', 'laptop', 'mouse', 'remote', 'keyboard', 'cell phone',
    'microwave', 'oven', 'toaster', 'sink', 'refrigerator', 'N/A', 'book',
    'clock', 'vase', 'scissors', 'teddy bear', 'hair drier', 'toothbrush'
]

# Class definition for the model
class ObjectDetectionModel(object):
	'''
		The blackbox object detection model (Faster RCNN for those who want to know).
		Given an image as numpy array (3, H, W), it detects objects (generates their category ids and bounding boxes).
	'''
	# __init__ function
	def __init__(self):
		self.model = torchvision.models.detection.fasterrcnn_resnet50_fpn(pretrained=True)
		self.model.eval()

	# function for calling the faster-rcnn model
	def __call__(self, input):
		'''
			Arguments:
				input (numpy array): A (3, H, W) array of numbers in [0, 1] representing the image.

			Returns:
				pred_boxes (list): list of bounding boxes, [[x1 y1 x2 y2], ..] where (x1, y1) are the coordinates of the top left corner 
									and (x2, y2) are the coordinates of the bottom right corner.

				pred_class (list): list of predicted classes

				pred_score (list): list of the probability (confidence) of prediction of each of the bounding boxes
			Tip:
				You can print the outputs to get better clarity :)
		'''

		input_tensor = torch.from_numpy(input)
		input_tensor = input_tensor.type(torch.FloatTensor)
		input_tensor = input_tensor.unsqueeze(0)
		predictions = self.model(input_tensor)
		pred_class = [COCO_INSTANCE_CATEGORY_NAMES[i] for i in list(predictions[0]['labels'].numpy())] # Get the Prediction Score
		pred_boxes = [[(i[0], i[1]), (i[2], i[3])] for i in list(predictions[0]['boxes'].detach().numpy())] # Bounding boxes
		pred_score = list(predictions[0]['scores'].detach().numpy())
		
		return pred_boxes, pred_class, pred_score




import pathlib
from setuptools import setup

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# This call to setup() does all the work
setup(
    name="imgBoxDetector",
    version="1.0.3",
    description="A model to detect various objects in an image",
    long_description=README,
    long_description_content_type="text/markdown",
    url="",
    author="Anish Sofat",
    author_email="sofatanish16@gmail.com",
    license="GPLv3",
    classifiers=[
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
    ],
    packages=["imgBoxDetector","imgBoxDetector.data","imgBoxDetector.data.transforms","imgBoxDetector.analysis"],
    include_package_data=True,
    install_requires=["Pillow","numpy","matplotlib","torch","torchvision"],
)
